use bytes::Bytes;
use leptos::*;
use server_fn::codec::{ByteStream, Streaming};
use uuid::Uuid;

use crate::api::{ChannelMessage, ChatMessage};

#[server(name=TestServerFn, output = Streaming)]
pub async fn stream_chat(id: Uuid, history: Vec<ChatMessage>) -> Result<ByteStream, ServerFnError> {
    use tokio::sync::mpsc;
    use tokio_stream::{wrappers::ReceiverStream, StreamExt};

    use crate::server::backends::{llama_chat, ChatService};

    let (tx, rx) = mpsc::channel::<ChannelMessage>(8);

    let service = llama_chat::LlamaService::new(id);

    let _handle = tokio::spawn(service.chat_request(history, tx));

    let broadcast_receive_stream = ReceiverStream::new(rx)
        .map(|msg| {
            tracing::trace!(?msg);
            serde_json::to_string(&msg).unwrap()
        })
        .map(|string| {
            let bytes = Bytes::from(string.as_bytes().to_vec());
            Ok(bytes)
        });
    let text_stream = ByteStream::new(broadcast_receive_stream);

    Ok(text_stream)
}
